import http from 'k6/http'
import { group, check, sleep } from 'k6'

import {
  projectsQuery,
  goodNewsArticlesQuery,
  volunteeringEngagementsQuery,
  lastInsightsQuery,
  featureTogglesQuery,
  userSuggestionsQuery,
} from './queries.js'
import { hasNewNotificationsQuery, authenticatedUserQuery } from '../common/queries.js'
import { headers, websiteUrl } from '../const.js'
import { addBatchRequest, checkResponse } from '../utils.js'
import { checkFailureRate, timeToFirstByte } from '../main.js'

export function home(data) {
  group('Homepage', function () {
    const res = http.get(websiteUrl, {
      headers,
    })

    const checkRes = check(res, {
      'is status 200': (r) => r.status === 200,
    })

    // Record check failures
    checkFailureRate.add(!checkRes)

    // Record time to first byte and tag it with the URL to be able to filter the results in Insights
    timeToFirstByte.add(res.timings.waiting, { ttfbURL: res.url })

    group('Unauthenticated user', function () {
      const responses = http.batch([
        addBatchRequest(
          projectsQuery,
          {
            offset: 0,
            limit: 5,
          },
          { name: 'ProjectsQuery' },
        ),
        addBatchRequest(goodNewsArticlesQuery, null, { name: 'GoodNewsArticlesQuery' }),
        addBatchRequest(
          volunteeringEngagementsQuery,
          {
            offset: 0,
            limit: 5,
          },
          { name: 'VolunteeringEngagementsQuery' },
        ),
        addBatchRequest(
          lastInsightsQuery,
          {
            offset: 0,
            limit: 4,
          },
          { name: 'LastInsightsQuery' },
        ),
        addBatchRequest(featureTogglesQuery, null, { name: 'FeatureTogglesQuery' }),
      ])

      checkResponse(responses[0])
      checkResponse(responses[1])
      checkResponse(responses[2])
      checkResponse(responses[3])
      checkResponse(responses[4])
      sleep(Math.random())
    })

    group('Authenticated user', function () {
      const responses = http.batch([
        addBatchRequest(
          projectsQuery,
          {
            offset: 0,
            limit: 5,
          },
          { name: 'ProjectsQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          goodNewsArticlesQuery,
          null,
          { name: 'GoodNewsArticlesQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          volunteeringEngagementsQuery,
          {
            offset: 0,
            limit: 5,
          },
          { name: 'VolunteeringEngagementsQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          lastInsightsQuery,
          {
            offset: 0,
            limit: 4,
          },
          { name: 'LastInsightsQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          featureTogglesQuery,
          null,
          { name: 'FeatureTogglesQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          authenticatedUserQuery,
          null,
          { name: 'AuthenticatedUserQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          userSuggestionsQuery,
          { limit: 10 },
          { name: 'UserSuggestionsQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(
          hasNewNotificationsQuery,
          null,
          { name: 'HasNewNotifcationsQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
      ])

      checkResponse(responses[0])
      checkResponse(responses[1])
      checkResponse(responses[2])
      checkResponse(responses[3])
      checkResponse(responses[4])
      checkResponse(responses[5])
      checkResponse(responses[6])
      checkResponse(responses[7])
      sleep(Math.random())
    })
  })
}
